package java.awt;

import sun.misc.SharedSecrets;
import java.lang.reflect.InvocationTargetException;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import sun.awt.AWTAccessor;
import java.awt.event.InputMethodEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.FocusEvent;
import java.util.EmptyStackException;
import java.awt.event.InvocationEvent;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.awt.dnd.SunDropTargetEvent;
import java.awt.event.MouseEvent;
import java.awt.peer.ComponentPeer;
import java.awt.event.PaintEvent;
import sun.awt.EventQueueItem;
import sun.awt.PeerEvent;
import sun.awt.AWTAutoShutdown;
import sun.awt.SunToolkit;
import sun.misc.JavaSecurityAccess;
import sun.util.logging.PlatformLogger;
import sun.awt.FwDispatcher;
import sun.awt.AppContext;
import java.lang.ref.WeakReference;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.atomic.AtomicInteger;

public class EventQueue
{
    private static final AtomicInteger threadInitNumber;
    private static final int LOW_PRIORITY = 0;
    private static final int NORM_PRIORITY = 1;
    private static final int HIGH_PRIORITY = 2;
    private static final int ULTIMATE_PRIORITY = 3;
    private static final int NUM_PRIORITIES = 4;
    private Queue[] queues;
    private EventQueue nextQueue;
    private EventQueue previousQueue;
    private final Lock pushPopLock;
    private final Condition pushPopCond;
    private static final Runnable dummyRunnable;
    private EventDispatchThread dispatchThread;
    private final ThreadGroup threadGroup;
    private final ClassLoader classLoader;
    private long mostRecentEventTime;
    private long mostRecentKeyEventTime;
    private WeakReference<AWTEvent> currentEvent;
    private volatile int waitForID;
    private final AppContext appContext;
    private final String name;
    private FwDispatcher fwDispatcher;
    private static volatile PlatformLogger eventLog;
    private static final int PAINT = 0;
    private static final int UPDATE = 1;
    private static final int MOVE = 2;
    private static final int DRAG = 3;
    private static final int PEER = 4;
    private static final int CACHE_LENGTH = 5;
    private static final JavaSecurityAccess javaSecurityAccess;
    
    private static final PlatformLogger getEventLog() {
        if (EventQueue.eventLog == null) {
            EventQueue.eventLog = PlatformLogger.getLogger("java.awt.event.EventQueue");
        }
        return EventQueue.eventLog;
    }
    
    public EventQueue() {
        this.queues = new Queue[4];
        this.threadGroup = Thread.currentThread().getThreadGroup();
        this.classLoader = Thread.currentThread().getContextClassLoader();
        this.mostRecentEventTime = System.currentTimeMillis();
        this.mostRecentKeyEventTime = System.currentTimeMillis();
        this.name = "AWT-EventQueue-" + EventQueue.threadInitNumber.getAndIncrement();
        for (int i = 0; i < 4; ++i) {
            this.queues[i] = new Queue();
        }
        this.appContext = AppContext.getAppContext();
        this.pushPopLock = (Lock)this.appContext.get(AppContext.EVENT_QUEUE_LOCK_KEY);
        this.pushPopCond = (Condition)this.appContext.get(AppContext.EVENT_QUEUE_COND_KEY);
    }
    
    public void postEvent(AWTEvent awtEvent) {
        SunToolkit.flushPendingEvents(this.appContext);
        this.postEventPrivate(awtEvent);
    }
    
    private final void postEventPrivate(AWTEvent awtEvent) {
        awtEvent.isPosted = true;
        this.pushPopLock.lock();
        try {
            if (this.nextQueue != null) {
                this.nextQueue.postEventPrivate(awtEvent);
                return;
            }
            if (this.dispatchThread == null) {
                if (awtEvent.getSource() == AWTAutoShutdown.getInstance()) {
                    return;
                }
                this.initDispatchThread();
            }
            this.postEvent(awtEvent, getPriority(awtEvent));
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    private static int getPriority(AWTEvent awtEvent) {
        if ((awtEvent instanceof PeerEvent)) {
            PeerEvent peerEvent = (PeerEvent)awtEvent;
            if ((peerEvent.getFlags() & 0x2L) != 0x0L) {
                return 3;
            }
            if ((peerEvent.getFlags() & 0x1L) != 0x0L) {
                return 2;
            }
            if ((peerEvent.getFlags() & 0x4L) != 0x0L) {
                return 0;
            }
        }
        int id = awtEvent.getID();
        if (id >= 800 && id <= 801) {
            return 0;
        }
        return 1;
    }
    
    private void postEvent(AWTEvent evt, int n) {
        if (this.coalesceEvent(evt, n)) {
            return;
        }
        EventQueueItem eventQueueItem = new EventQueueItem(evt);
        this.cacheEQItem(eventQueueItem);
        int n2 = evt.getID() == this.waitForID;
        if (this.queues[n].head == null) {
            int noEvents = this.noEvents();
            Queue queue = this.queues[n];
            this.queues[n].tail = eventQueueItem;
            queue.head = eventQueueItem;
            if (!(noEvents == 0)) {
                if (evt.getSource() != AWTAutoShutdown.getInstance()) {
                    AWTAutoShutdown.getInstance().notifyThreadBusy(this.dispatchThread);
                }
                this.pushPopCond.signalAll();
            }
            else if (!(n2 == 0)) {
                this.pushPopCond.signalAll();
            }
        }
        else {
            this.queues[n].tail.next = eventQueueItem;
            this.queues[n].tail = eventQueueItem;
            if (!(n2 == 0)) {
                this.pushPopCond.signalAll();
            }
        }
        return;
    }
    
    private boolean coalescePaintEvent(PaintEvent paintEvent) {
        ComponentPeer peer = ((Component)paintEvent.getSource()).peer;
        if (peer != null) {
            peer.coalescePaintEvent(paintEvent);
        }
        EventQueueItem[] eventCache = ((Component)paintEvent.getSource()).eventCache;
        if (eventCache == null) {
            return false;
        }
        int eventToCacheIndex = eventToCacheIndex(paintEvent);
        if (eventToCacheIndex != -1 && eventCache[eventToCacheIndex] != null) {
            PaintEvent mergePaintEvents = this.mergePaintEvents(paintEvent, (PaintEvent)eventCache[eventToCacheIndex].event);
            if (mergePaintEvents != null) {
                eventCache[eventToCacheIndex].event = mergePaintEvents;
                return true;
            }
        }
        return false;
    }
    
    private PaintEvent mergePaintEvents(PaintEvent paintEvent, PaintEvent paintEvent2) {
        Rectangle updateRect = paintEvent.getUpdateRect();
        Rectangle updateRect2 = paintEvent2.getUpdateRect();
        if (updateRect2.contains(updateRect)) {
            return paintEvent2;
        }
        if (updateRect.contains(updateRect2)) {
            return paintEvent;
        }
        return null;
    }
    
    private boolean coalesceMouseEvent(MouseEvent event) {
        EventQueueItem[] eventCache = ((Component)event.getSource()).eventCache;
        if (eventCache == null) {
            return false;
        }
        int eventToCacheIndex = eventToCacheIndex(event);
        if (eventToCacheIndex != -1 && eventCache[eventToCacheIndex] != null) {
            eventCache[eventToCacheIndex].event = event;
            return true;
        }
        return false;
    }
    
    private boolean coalescePeerEvent(PeerEvent coalesceEvents) {
        EventQueueItem[] eventCache = ((Component)coalesceEvents.getSource()).eventCache;
        if (eventCache == null) {
            return false;
        }
        int eventToCacheIndex = eventToCacheIndex(coalesceEvents);
        if (eventToCacheIndex != -1 && eventCache[eventToCacheIndex] != null) {
            coalesceEvents = coalesceEvents.coalesceEvents((PeerEvent)eventCache[eventToCacheIndex].event);
            if (coalesceEvents != null) {
                eventCache[eventToCacheIndex].event = coalesceEvents;
                return true;
            }
            eventCache[eventToCacheIndex] = null;
        }
        return false;
    }
    
    private boolean coalesceOtherEvent(AWTEvent awtEvent, int n) {
        int id = awtEvent.getID();
        Component component = (Component)awtEvent.getSource();
        for (EventQueueItem eventQueueItem = this.queues[n].head; eventQueueItem != null; eventQueueItem = eventQueueItem.next) {
            if (eventQueueItem.event.getSource() == component && eventQueueItem.event.getID() == id) {
                AWTEvent coalesceEvents = component.coalesceEvents(eventQueueItem.event, awtEvent);
                if (coalesceEvents != null) {
                    eventQueueItem.event = coalesceEvents;
                    return true;
                }
            }
        }
        return false;
    }
    
    private boolean coalesceEvent(AWTEvent awtEvent, int n) {
        if (!(awtEvent.getSource() instanceof Component)) {
            return false;
        }
        if ((awtEvent instanceof PeerEvent)) {
            return this.coalescePeerEvent((PeerEvent)awtEvent);
        }
        if (((Component)awtEvent.getSource()).isCoalescingEnabled() && this.coalesceOtherEvent(awtEvent, n)) {
            return true;
        }
        if ((awtEvent instanceof PaintEvent)) {
            return this.coalescePaintEvent((PaintEvent)awtEvent);
        }
        return awtEvent instanceof MouseEvent && this.coalesceMouseEvent((MouseEvent)awtEvent);
    }
    
    private void cacheEQItem(EventQueueItem eventQueueItem) {
        int eventToCacheIndex = eventToCacheIndex(eventQueueItem.event);
        if (eventToCacheIndex != -1 && eventQueueItem.event.getSource() instanceof Component) {
            Component component = (Component)eventQueueItem.event.getSource();
            if (component.eventCache == null) {
                component.eventCache = new EventQueueItem[5];
            }
            component.eventCache[eventToCacheIndex] = eventQueueItem;
        }
    }
    
    private void uncacheEQItem(EventQueueItem eventQueueItem) {
        int eventToCacheIndex = eventToCacheIndex(eventQueueItem.event);
        if (eventToCacheIndex != -1 && eventQueueItem.event.getSource() instanceof Component) {
            Component component = (Component)eventQueueItem.event.getSource();
            if (component.eventCache == null) {
                return;
            }
            component.eventCache[eventToCacheIndex] = null;
        }
    }
    
    private static int eventToCacheIndex(AWTEvent awtEvent) {
        switch (awtEvent.getID()) {
            case 800: {
                return 0;
            }
            case 801: {
                return 1;
            }
            case 503: {
                return 2;
            }
            case 506: {
                return (awtEvent instanceof SunDropTargetEvent) ? -1 : 3;
            }
            default: {
                return (awtEvent instanceof PeerEvent) ? 4 : -1;
            }
        }
    }
    
    private boolean noEvents() {
        int i = 0;
        while (i < 4) {
            if (this.queues[i].head != null) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public AWTEvent getNextEvent() throws InterruptedException {
        while (true) {
            SunToolkit.flushPendingEvents(this.appContext);
            this.pushPopLock.lock();
            try {
                AWTEvent nextEventPrivate = this.getNextEventPrivate();
                if (nextEventPrivate != null) {
                    return nextEventPrivate;
                }
                AWTAutoShutdown.getInstance().notifyThreadFree(this.dispatchThread);
                this.pushPopCond.await();
            }
            finally {
                this.pushPopLock.unlock();
            }
        }
    }
    
    AWTEvent getNextEventPrivate() throws InterruptedException {
        int i = 3;
        while (i >= 0) {
            if (this.queues[i].head != null) {
                EventQueueItem head = this.queues[i].head;
                this.queues[i].head = head.next;
                if (head.next == null) {
                    this.queues[i].tail = null;
                }
                this.uncacheEQItem(head);
                return head.event;
            }
            else {
                --i;
            }
        }
        return null;
    }
    
    AWTEvent getNextEvent(int waitForID) throws InterruptedException {
        while (true) {
            SunToolkit.flushPendingEvents(this.appContext);
            this.pushPopLock.lock();
            try {
                for (int i = 0; i < 4; ++i) {
                    EventQueueItem eventQueueItem = this.queues[i].head;
                    EventQueueItem tail = null;
                    while (eventQueueItem != null) {
                        if (eventQueueItem.event.getID() == waitForID) {
                            if (tail == null) {
                                this.queues[i].head = eventQueueItem.next;
                            }
                            else {
                                tail.next = eventQueueItem.next;
                            }
                            if (this.queues[i].tail == eventQueueItem) {
                                this.queues[i].tail = tail;
                            }
                            this.uncacheEQItem(eventQueueItem);
                            return eventQueueItem.event;
                        }
                        else {
                            tail = eventQueueItem;
                            eventQueueItem = eventQueueItem.next;
                        }
                    }
                }
                this.waitForID = waitForID;
                this.pushPopCond.await();
                this.waitForID = 0;
            }
            finally {
                this.pushPopLock.unlock();
            }
        }
    }
    
    public AWTEvent peekEvent() {
        this.pushPopLock.lock();
        try {
            int i = 3;
            while (i >= 0) {
                if (this.queues[i].head != null) {
                    return this.queues[i].head.event;
                }
                --i;
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
        return null;
    }
    
    public AWTEvent peekEvent(int n) {
        this.pushPopLock.lock();
        try {
            for (int i = 3; i >= 0; --i) {
                EventQueueItem eventQueueItem = this.queues[i].head;
                while (eventQueueItem != null) {
                    if (eventQueueItem.event.getID() == n) {
                        return eventQueueItem.event;
                    }
                    eventQueueItem = eventQueueItem.next;
                }
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
        return null;
    }
    
    protected void dispatchEvent(final AWTEvent awtEvent) {
        final Object source = awtEvent.getSource();
        final PrivilegedAction<Void> privilegedAction = new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                if (EventQueue.this.fwDispatcher == null || EventQueue.this.isDispatchThreadImpl()) {
                    EventQueue.access$500(EventQueue.this, awtEvent, source);
                }
                else {
                    EventQueue.this.fwDispatcher.scheduleDispatch(new Runnable() {
                        @Override
                        public void run() {
                            if (EventQueue.this.dispatchThread.filterAndCheckEvent(awtEvent)) {
                                EventQueue.access$500(EventQueue.this, awtEvent, source);
                            }
                        }
                    });
                }
                return null;
            }
        };
        AccessControlContext context = AccessController.getContext();
        AccessControlContext accessControlContext = getAccessControlContextFrom(source);
        final AccessControlContext accessControlContext2 = awtEvent.getAccessControlContext();
        if (accessControlContext == null) {
            EventQueue.javaSecurityAccess.doIntersectionPrivilege(privilegedAction, context, accessControlContext2);
        }
        else {
            EventQueue.javaSecurityAccess.doIntersectionPrivilege(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    EventQueue.javaSecurityAccess.doIntersectionPrivilege(privilegedAction, accessControlContext2);
                    return null;
                }
            }, context, accessControlContext);
        }
    }
    
    private static AccessControlContext getAccessControlContextFrom(Object o) {
        return (o instanceof Component) ? ((Component)o).getAccessControlContext() : ((o instanceof MenuComponent) ? ((MenuComponent)o).getAccessControlContext() : ((o instanceof TrayIcon) ? ((TrayIcon)o).getAccessControlContext() : null));
    }
    
    private void dispatchEventImpl(AWTEvent awtEvent, Object o) {
        awtEvent.isPosted = true;
        if ((awtEvent instanceof ActiveEvent)) {
            this.setCurrentEventAndMostRecentTimeImpl(awtEvent);
            ((ActiveEvent)awtEvent).dispatch();
        }
        else if ((o instanceof Component)) {
            ((Component)o).dispatchEvent(awtEvent);
            awtEvent.dispatched();
        }
        else if ((o instanceof MenuComponent)) {
            ((MenuComponent)o).dispatchEvent(awtEvent);
        }
        else if ((o instanceof TrayIcon)) {
            ((TrayIcon)o).dispatchEvent(awtEvent);
        }
        else if ((o instanceof AWTAutoShutdown)) {
            if (this.noEvents()) {
                this.dispatchThread.stopDispatching();
            }
        }
        else if (getEventLog().isLoggable(PlatformLogger.Level.FINE)) {
            getEventLog().fine("Unable to dispatch event: " + awtEvent);
        }
    }
    
    public static long getMostRecentEventTime() {
        return Toolkit.getEventQueue().getMostRecentEventTimeImpl();
    }
    
    private long getMostRecentEventTimeImpl() {
        this.pushPopLock.lock();
        try {
            return (Thread.currentThread() == this.dispatchThread) ? this.mostRecentEventTime : System.currentTimeMillis();
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    long getMostRecentEventTimeEx() {
        this.pushPopLock.lock();
        try {
            return this.mostRecentEventTime;
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    public static AWTEvent getCurrentEvent() {
        return Toolkit.getEventQueue().getCurrentEventImpl();
    }
    
    private AWTEvent getCurrentEventImpl() {
        this.pushPopLock.lock();
        try {
            return (Thread.currentThread() == this.dispatchThread) ? ((AWTEvent)this.currentEvent.get()) : null;
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    public void push(EventQueue eventQueue) {
        if (getEventLog().isLoggable(PlatformLogger.Level.FINE)) {
            getEventLog().fine("EventQueue.push(" + eventQueue + ")");
        }
        this.pushPopLock.lock();
        try {
            while (this.nextQueue != null) {
                this = this.nextQueue;
            }
            if (this.fwDispatcher != null) {
                throw new RuntimeException("push() to queue with fwDispatcher");
            }
            if (this.dispatchThread != null && this.dispatchThread.getEventQueue() == this) {
                eventQueue.dispatchThread = this.dispatchThread;
                this.dispatchThread.setEventQueue(eventQueue);
            }
            while (this.peekEvent() != null) {
                try {
                    eventQueue.postEventPrivate(this.getNextEventPrivate());
                }
                catch (InterruptedException t) {
                    if (getEventLog().isLoggable(PlatformLogger.Level.FINE)) {
                        getEventLog().fine("Interrupted push", t);
                    }
                    else {
                        continue;
                    }
                }
            }
            if (this.dispatchThread != null) {
                this.postEventPrivate(new InvocationEvent(this, EventQueue.dummyRunnable));
            }
            eventQueue.previousQueue = this;
            this.nextQueue = eventQueue;
            if (this.appContext.get(AppContext.EVENT_QUEUE_KEY) == this) {
                this.appContext.put(AppContext.EVENT_QUEUE_KEY, eventQueue);
            }
            this.pushPopCond.signalAll();
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    protected void pop() throws EmptyStackException {
        if (getEventLog().isLoggable(PlatformLogger.Level.FINE)) {
            getEventLog().fine("EventQueue.pop(" + this + ")");
        }
        this.pushPopLock.lock();
        try {
            while (this.nextQueue != null) {
                this = this.nextQueue;
            }
            EventQueue previousQueue = this.previousQueue;
            if (previousQueue == null) {
                throw new EmptyStackException();
            }
            this.previousQueue = null;
            previousQueue.nextQueue = null;
            while (this.peekEvent() != null) {
                try {
                    previousQueue.postEventPrivate(this.getNextEventPrivate());
                }
                catch (InterruptedException t) {
                    if (getEventLog().isLoggable(PlatformLogger.Level.FINE)) {
                        getEventLog().fine("Interrupted pop", t);
                    }
                    else {
                        continue;
                    }
                }
            }
            if (this.dispatchThread != null && this.dispatchThread.getEventQueue() == this) {
                previousQueue.dispatchThread = this.dispatchThread;
                this.dispatchThread.setEventQueue(previousQueue);
            }
            if (this.appContext.get(AppContext.EVENT_QUEUE_KEY) == this) {
                this.appContext.put(AppContext.EVENT_QUEUE_KEY, previousQueue);
            }
            this.postEventPrivate(new InvocationEvent(this, EventQueue.dummyRunnable));
            this.pushPopCond.signalAll();
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    public SecondaryLoop createSecondaryLoop() {
        return this.createSecondaryLoop(null, null, 0L);
    }
    
    SecondaryLoop createSecondaryLoop(Conditional conditional, EventFilter eventFilter, long n) {
        this.pushPopLock.lock();
        try {
            if (this.nextQueue != null) {
                return this.nextQueue.createSecondaryLoop(conditional, eventFilter, n);
            }
            if (this.fwDispatcher != null) {
                return new EventQueue.FwSecondaryLoopWrapper(this.fwDispatcher.createSecondaryLoop(), eventFilter);
            }
            if (this.dispatchThread == null) {
                this.initDispatchThread();
            }
            return new WaitDispatchSupport(this.dispatchThread, conditional, eventFilter, n);
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    public static boolean isDispatchThread() {
        return Toolkit.getEventQueue().isDispatchThreadImpl();
    }
    
    final boolean isDispatchThreadImpl() {
        this.pushPopLock.lock();
        try {
            for (EventQueue eventQueue = this.nextQueue; eventQueue != null; eventQueue = this.nextQueue) {
                this = eventQueue;
            }
            if (this.fwDispatcher != null) {
                return this.fwDispatcher.isDispatchThread();
            }
            return Thread.currentThread() == this.dispatchThread;
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    final void initDispatchThread() {
        this.pushPopLock.lock();
        try {
            if (this.dispatchThread == null && !this.threadGroup.isDestroyed() && !this.appContext.isDisposed()) {
                (this.dispatchThread = (EventDispatchThread)AccessController.doPrivileged(new PrivilegedAction<EventDispatchThread>() {
                    @Override
                    public EventDispatchThread run() {
                        EventDispatchThread thread = new EventDispatchThread(EventQueue.this.threadGroup, EventQueue.this.name, EventQueue.this);
                        thread.setContextClassLoader(EventQueue.this.classLoader);
                        thread.setPriority(6);
                        thread.setDaemon(false);
                        AWTAutoShutdown.getInstance().notifyThreadBusy(thread);
                        return thread;
                    }
                })).start();
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    final void detachDispatchThread(EventDispatchThread thread) {
        SunToolkit.flushPendingEvents(this.appContext);
        this.pushPopLock.lock();
        try {
            if (thread == this.dispatchThread) {
                this.dispatchThread = null;
            }
            AWTAutoShutdown.getInstance().notifyThreadFree(thread);
            if (this.peekEvent() != null) {
                this.initDispatchThread();
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    final EventDispatchThread getDispatchThread() {
        this.pushPopLock.lock();
        try {
            return this.dispatchThread;
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    final void removeSourceEvents(Object o, boolean b) {
        SunToolkit.flushPendingEvents(this.appContext);
        this.pushPopLock.lock();
        try {
            EventQueueItem tail;
            for (int i = 0; i < 4; this.queues[i].tail = tail, ++i) {
                EventQueueItem eventQueueItem = this.queues[i].head;
                tail = null;
                while (eventQueueItem != null) {
                    if (eventQueueItem.event.getSource() == o && (b || (!(eventQueueItem.event instanceof SequencedEvent) && !(eventQueueItem.event instanceof SentEvent) && !(eventQueueItem.event instanceof FocusEvent) && !(eventQueueItem.event instanceof WindowEvent) && !(eventQueueItem.event instanceof KeyEvent) && !(eventQueueItem.event instanceof InputMethodEvent)))) {
                        if ((eventQueueItem.event instanceof SequencedEvent)) {
                            ((SequencedEvent)eventQueueItem.event).dispose();
                        }
                        if ((eventQueueItem.event instanceof SentEvent)) {
                            ((SentEvent)eventQueueItem.event).dispose();
                        }
                        if ((eventQueueItem.event instanceof InvocationEvent)) {
                            AWTAccessor.getInvocationEventAccessor().dispose((InvocationEvent)eventQueueItem.event);
                        }
                        if (tail == null) {
                            this.queues[i].head = eventQueueItem.next;
                        }
                        else {
                            tail.next = eventQueueItem.next;
                        }
                        this.uncacheEQItem(eventQueueItem);
                    }
                    else {
                        tail = eventQueueItem;
                    }
                    eventQueueItem = eventQueueItem.next;
                }
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    synchronized long getMostRecentKeyEventTime() {
        this.pushPopLock.lock();
        try {
            return this.mostRecentKeyEventTime;
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    static void setCurrentEventAndMostRecentTime(AWTEvent currentEventAndMostRecentTimeImpl) {
        Toolkit.getEventQueue().setCurrentEventAndMostRecentTimeImpl(currentEventAndMostRecentTimeImpl);
    }
    
    private void setCurrentEventAndMostRecentTimeImpl(AWTEvent referent) {
        this.pushPopLock.lock();
        try {
            if (Thread.currentThread() != this.dispatchThread) {
                return;
            }
            this.currentEvent = new WeakReference(referent);
            long b = Long.MIN_VALUE;
            if ((referent instanceof InputEvent)) {
                InputEvent inputEvent = (InputEvent)referent;
                b = inputEvent.getWhen();
                if ((referent instanceof KeyEvent)) {
                    this.mostRecentKeyEventTime = inputEvent.getWhen();
                }
            }
            else if ((referent instanceof InputMethodEvent)) {
                b = ((InputMethodEvent)referent).getWhen();
            }
            else if ((referent instanceof ActionEvent)) {
                b = ((ActionEvent)referent).getWhen();
            }
            else if ((referent instanceof InvocationEvent)) {
                b = ((InvocationEvent)referent).getWhen();
            }
            this.mostRecentEventTime = Math.max(this.mostRecentEventTime, b);
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    public static void invokeLater(Runnable runnable) {
        Toolkit.getEventQueue().postEvent(new InvocationEvent(Toolkit.getDefaultToolkit(), runnable));
    }
    
    public static void invokeAndWait(Runnable runnable) throws InterruptedException, InvocationTargetException {
        invokeAndWait(Toolkit.getDefaultToolkit(), runnable);
    }
    
    static void invokeAndWait(Object source, Runnable runnable) throws InterruptedException, InvocationTargetException {
        if (isDispatchThread()) {
            throw new Error("Cannot call invokeAndWait from the event dispatcher thread");
        }
        class AWTInvocationLock
        {
        }
        EventQueue.1AWTInvocationLock notifier = new AWTInvocationLock();
        InvocationEvent invocationEvent = new InvocationEvent(source, runnable, notifier, true);
        synchronized (notifier) {
            Toolkit.getEventQueue().postEvent(invocationEvent);
            while (!invocationEvent.isDispatched()) {
                notifier.wait();
            }
        }
        Throwable throwable = invocationEvent.getThrowable();
        if (throwable != null) {
            throw new InvocationTargetException(throwable);
        }
        return;
    }
    
    private void wakeup(boolean b) {
        this.pushPopLock.lock();
        try {
            if (this.nextQueue != null) {
                this.nextQueue.wakeup(b);
            }
            else if (this.dispatchThread != null) {
                this.pushPopCond.signalAll();
            }
            else if (!b) {
                this.initDispatchThread();
            }
        }
        finally {
            this.pushPopLock.unlock();
        }
    }
    
    private void setFwDispatcher(FwDispatcher fwDispatcher) {
        if (this.nextQueue != null) {
            this.nextQueue.setFwDispatcher(fwDispatcher);
        }
        else {
            this.fwDispatcher = fwDispatcher;
        }
    }
    
    static boolean access$000(EventQueue eventQueue) {
        return eventQueue.noEvents();
    }
    
    static void access$100(EventQueue eventQueue, boolean b) {
        eventQueue.wakeup(b);
    }
    
    static void access$200(EventQueue eventQueue, FwDispatcher fwDispatcher) {
        eventQueue.setFwDispatcher(fwDispatcher);
    }
    
    static long access$300(EventQueue eventQueue) {
        return eventQueue.getMostRecentEventTimeImpl();
    }
    
    static void access$500(EventQueue eventQueue, AWTEvent awtEvent, Object o) {
        eventQueue.dispatchEventImpl(awtEvent, o);
    }
    
    static {
        threadInitNumber = new AtomicInteger(0);
        dummyRunnable = new Runnable() {
            @Override
            public void run() {
            }
        };
        AWTAccessor.setEventQueueAccessor(new AWTAccessor.EventQueueAccessor() {
            @Override
            public Thread getDispatchThread(EventQueue eventQueue) {
                return eventQueue.getDispatchThread();
            }
            
            @Override
            public boolean isDispatchThreadImpl(EventQueue eventQueue) {
                return eventQueue.isDispatchThreadImpl();
            }
            
            @Override
            public void removeSourceEvents(EventQueue eventQueue, Object o, boolean b) {
                eventQueue.removeSourceEvents(o, b);
            }
            
            @Override
            public boolean noEvents(EventQueue eventQueue) {
                return EventQueue.access$000(eventQueue);
            }
            
            @Override
            public void wakeup(EventQueue eventQueue, boolean b) {
                EventQueue.access$100(eventQueue, b);
            }
            
            @Override
            public void invokeAndWait(Object o, Runnable runnable) throws InterruptedException, InvocationTargetException {
                EventQueue.invokeAndWait(o, runnable);
            }
            
            @Override
            public void setFwDispatcher(EventQueue eventQueue, FwDispatcher fwDispatcher) {
                EventQueue.access$200(eventQueue, fwDispatcher);
            }
            
            @Override
            public long getMostRecentEventTime(EventQueue eventQueue) {
                return EventQueue.access$300(eventQueue);
            }
        });
        javaSecurityAccess = SharedSecrets.getJavaSecurityAccess();
    }
    
    private class FwSecondaryLoopWrapper implements SecondaryLoop
    {
        private final SecondaryLoop loop;
        private final EventFilter filter;
        
        public FwSecondaryLoopWrapper(SecondaryLoop loop, EventFilter filter) {
            this.loop = loop;
            this.filter = filter;
        }
        
        @Override
        public boolean enter() {
            if (this.filter != null) {
                EventQueue.this.dispatchThread.addEventFilter(this.filter);
            }
            return this.loop.enter();
        }
        
        @Override
        public boolean exit() {
            if (this.filter != null) {
                EventQueue.this.dispatchThread.removeEventFilter(this.filter);
            }
            return this.loop.exit();
        }
    }
}
